<?php
class category 
{
    /**
    * Detailed errors of a class (for the programmer and log-files)
    * array('error type (1 - fatal (write log), 2 - fatal (write log, send email)',
    * 'error info string', 'function', 'info 1', 'info 2').
    * 
    * @var array
    */
    var $ERRORS = array();

    /**
    * The information on a error for the user
    * array('string (error information)').
    * 
    * @var array
    */
    var $ERRORS_MES = array();

    /**
    * Name of the table where tree is stored.
    * 
    * @var string
    */
    var $table;

    /**
    * Unique number of node.
    * 
    * @var bigint
    */
    var $table_id;

    /**
    * @var integer
    */
    var $table_left;

    /**
    * @var integer
    */
    var $table_right;

    /**
    * Level of nesting.
    * 
    * @var integer
    */
    var $table_level;
    
    /**
    * Parent_id.
    * 
    * @var integer
    */
    var $table_parent;

    /**
    * DB resource object.
    * 
    * @var object
    */
    var $res;

    /**
    * Databse layer object.
    * 
    * @var object
    */
    var $db;
    
    var $tree_error_code;

    /**
    * The class constructor: initializes dbtree variables.
    * 
    * @param string $table Name of the table
    * @param string $prefix A prefix for fields of the table(the example, mytree_id. 'mytree' is prefix)
    * @param object $db
    * @return object
    */
    function category($table, $prefix, &$db)
    {
        $this->db = &$db;
        $this->table = $table;
        $this->table_id = $prefix . '_id';
        $this->table_left = $prefix . '_left';
        $this->table_sort = $prefix . '_sort';
        $this->table_right = $prefix . '_right';
        $this->table_level = $prefix . '_level';
        $this->table_parent = $prefix . '_parent';
        $this->tree_error_code = '0';
        unset($prefix, $table);
    }

    /**
    * Sets initial parameters of a tree and creates root of tree
    * ATTENTION, all previous values in table are destroyed.
    * 
    * @param array $data Contains parameters for additional fields of a tree (if is): 'filed name' => 'importance'
    * @return id first element if successful, FALSE otherwise.
    */
    function Clear($data = array()) 
    {
        $sql = 'TRUNCATE ' . $this->table;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $sql = 'DELETE FROM ' . $this->table;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        foreach ($data as $k => $v) 
        {
        	if ($v == '')
        	{
        		unset($data[$k]);
        	}
		}
        if (!empty($data)) 
        {
            $fld_names = implode(', ', array_keys($data)) . ', ';
            $fld_values = '\'' . implode('\', \'', array_values($data)) . '\', ';
        }
        $fld_names .= $this->table_left . ', ' . $this->table_right . ', ' . $this->table_level . ', ' . $this->table_parent;
        $fld_values .= '1, 2, 1, 0';
        $id = $this->db->GenID($this->table . '_seq', 2);
        $sql = 'INSERT INTO ' . $this->table . ' (' . $this->table_id . ', ' . $fld_names . ') VALUES (' . $id . ', ' . $fld_values . ')';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        return $id;
    }

    /**
    * Receives left, right and level, parent for unit with number id.
    *
    * @param integer $section_id Unique section id
    * @param integer $cache Recordset is cached for $cache microseconds
    * @return array - left, right, level, parent
    */
    function GetNodeInfo($section_id) 
    {
        $sql = 'SELECT ' . $this->table_left . ', ' . $this->table_right . ', ' . $this->table_level . ', ' . $this->table_parent . ' FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = ' . (int)$section_id;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        if (0 == $res->RecordCount()) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('no_element_in_tree') : 'no_element_in_tree';
            return FALSE;
        }
        $data = $res->FetchRow();
        unset($res);
        return array($data[$this->table_left], $data[$this->table_right], $data[$this->table_level], $data[$this->table_parent]);
    }
    
    function GetNodeInfoAll($id, $fields) 
    {
    	$sql = $this->PrepareSel($fields);
        $sql .= ' WHERE ' . $this->table_id . ' = ' . (int)$id;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        if (0 == $res->RecordCount()) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('no_element_in_tree') : 'no_element_in_tree';
            return FALSE;
        }
        $data = $res->FetchRow();
        unset($res);
        return $data;
    }

    /**
    * Receives parent left, right, level, parent for unit with number $id.
    *
    * @param integer $section_id
    * @param integer $cache Recordset is cached for $cache microseconds
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @return array - with (all information)
    */
    function GetParentInfoAll($id, $fields)
    {
        $node_info = $this->GetNodeInfo($id);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        $sql = $this->PrepareSel($fields);
        $sql .= ' WHERE ' . $this->table_id . ' = ' . $parent;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        if (0 == $res->RecordCount()) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('no_element_in_tree') : 'no_element_in_tree';
            return FALSE;
        }
        $data = $res->FetchRow();
        unset($res);
        return $data;
    }


    /**
    * Add a new element in the tree to element with number $section_id.
    *
    * @param integer $section_id Number of a parental element
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $data Contains parameters for additional fields of a tree (if is): array('filed name' => 'importance', etc)
    * @return integer Inserted element id
    */
    function Insert($section_id, $condition = '', $data = array()) 
    {
        $node_info = $this->GetNodeInfo($section_id);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        $data[$this->table_left] = $rightId;
        $data[$this->table_right] = ($rightId + 1);
        $data[$this->table_level] = ($level + 1);
        $data[$this->table_parent] = $section_id;
        if (!empty($condition))
        {
            $condition = $this->_PrepareCondition($condition);
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . '=CASE WHEN ' . $this->table_left . '>' . $rightId . ' THEN ' . $this->table_left . '+2 ELSE ' . $this->table_left . ' END, '
        . $this->table_right . '=CASE WHEN ' . $this->table_right . '>=' . $rightId . ' THEN ' . $this->table_right . '+2 ELSE ' . $this->table_right . ' END '
        . 'WHERE ' . $this->table_right . '>=' . $rightId;
        $sql .= $condition;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = -1';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $data[$this->table_id] = $this->db->GenID($this->table . '_seq', 2); 
        $sql = $this->db->GetInsertSQL($res, $data);
        if (!empty($sql)) 
        {
            $res = $this->db->Execute($sql);
            if (FALSE === $res) 
            {
                $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
                $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
                $this->db->FailTrans();
                return FALSE;
            }
        }
        $this->db->CompleteTrans();
        return $data[$this->table_id];
    }

    /**
    * Add a new element in the tree near element with number id.
    *
    * @param integer $ID Number of a parental element
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $data Contains parameters for additional fields of a tree (if is): array('filed name' => 'importance', etc)
    * @return integer Inserted element id
    */
    function InsertNear($ID, $condition = '', $data = array())
    {
        $node_info = $this->GetNodeInfo($ID);
        if (FALSE === $node_info)
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        $data[$this->table_left] = ($rightId + 1);
        $data[$this->table_right] = ($rightId + 2);
        $data[$this->table_level] = ($level);
        $data[$this->table_parent] = ($parent);
        if (!empty($condition))
        {
            $condition = $this->_PrepareCondition($condition);
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' > ' . $rightId . ' THEN ' . $this->table_left . ' + 2 ELSE ' . $this->table_left . ' END, '
        . $this->table_right . ' = CASE WHEN ' . $this->table_right . '> ' . $rightId . ' THEN ' . $this->table_right . ' + 2 ELSE ' . $this->table_right . ' END '
        . 'WHERE ' . $this->table_right . ' > ' . $rightId;
        $sql .= $condition;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = -1';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $data[$this->table_id] = $this->db->GenID($this->table . '_seq', 2);
        $sql = $this->db->GetInsertSQL($res, $data);
        if (!empty($sql)) 
        {
            $res = $this->db->Execute($sql);
            if (FALSE === $res) 
            {
                $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
                $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
                $this->db->FailTrans();
                return FALSE;
            }
        }
        $this->db->CompleteTrans();
        return $data[$this->table_id];
    }

    /**
    * Assigns a node with all its children to another parent.
    *
    * @param integer $ID node ID
    * @param integer $newParentId ID of new parent node
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @return bool TRUE if successful, FALSE otherwise.
    */
    function MoveAll($ID, $newParentId, $condition = '') 
    {
        $node_info = $this->GetNodeInfo($ID);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId, $level) = $node_info;
        $node_info = $this->GetNodeInfo($newParentId);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftIdP, $rightIdP, $levelP) = $node_info;
        if ($ID == $newParentId || $leftId == $leftIdP || ($leftIdP >= $leftId && $leftIdP <= $rightId) || ($level == $levelP+1 && $leftId > $leftIdP && $rightId < $rightIdP)) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('cant_move_tree') : 'cant_move_tree';
            return FALSE;
        }
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition);
        }
        if ($leftIdP < $leftId && $rightIdP > $rightId && $levelP < $level - 1) 
        {
            $sql = 'UPDATE ' . $this->table . ' SET '
            . $this->table_level . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_level.sprintf('%+d', -($level-1)+$levelP) . ' ELSE ' . $this->table_level . ' END, '
            . $this->table_right . ' = CASE WHEN ' . $this->table_right . ' BETWEEN ' . ($rightId+1) . ' AND ' . ($rightIdP-1) . ' THEN ' . $this->table_right . '-' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_right . '+' . ((($rightIdP-$rightId-$level+$levelP)/2)*2+$level-$levelP-1) . ' ELSE ' . $this->table_right . ' END, '
            . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId+1) . ' AND ' . ($rightIdP-1) . ' THEN ' . $this->table_left . '-' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_left . '+' . ((($rightIdP-$rightId-$level+$levelP)/2)*2+$level-$levelP-1) . ' ELSE ' . $this->table_left . ' END '
            . 'WHERE ' . $this->table_left . ' BETWEEN ' . ($leftIdP+1) . ' AND ' . ($rightIdP-1);
        } elseif ($leftIdP < $leftId) 
        {
            $sql = 'UPDATE ' . $this->table . ' SET '
            . $this->table_level . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_level.sprintf('%+d', -($level-1)+$levelP) . ' ELSE ' . $this->table_level . ' END, '
            . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $rightIdP . ' AND ' . ($leftId-1) . ' THEN ' . $this->table_left . '+' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_left . '-' . ($leftId-$rightIdP) . ' ELSE ' . $this->table_left . ' END, '
            . $this->table_right . ' = CASE WHEN ' . $this->table_right . ' BETWEEN ' . $rightIdP . ' AND ' . $leftId . ' THEN ' . $this->table_right . '+' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_right . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_right . '-' . ($leftId-$rightIdP) . ' ELSE ' . $this->table_right . ' END '
            . 'WHERE (' . $this->table_left . ' BETWEEN ' . $leftIdP . ' AND ' . $rightId. ' '
            . 'OR ' . $this->table_right . ' BETWEEN ' . $leftIdP . ' AND ' . $rightId . ')';
        } else 
        {
            $sql = 'UPDATE ' . $this->table . ' SET '
            . $this->table_level . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_level.sprintf('%+d', -($level-1)+$levelP) . ' ELSE ' . $this->table_level . ' END, '
            . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $rightId . ' AND ' . $rightIdP . ' THEN ' . $this->table_left . '-' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_left . '+' . ($rightIdP-1-$rightId) . ' ELSE ' . $this->table_left . ' END, '
            . $this->table_right . ' = CASE WHEN ' . $this->table_right . ' BETWEEN ' . ($rightId+1) . ' AND ' . ($rightIdP-1) . ' THEN ' . $this->table_right . '-' . ($rightId-$leftId+1) . ' '
            . 'WHEN ' . $this->table_right . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_right . '+' . ($rightIdP-1-$rightId) . ' ELSE ' . $this->table_right . ' END '
            . 'WHERE (' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightIdP . ' '
            . 'OR ' . $this->table_right . ' BETWEEN ' . $leftId . ' AND ' . $rightIdP . ')';
        }
        $sql .= $condition;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_parent . ' = ' . $newParentId 
        . ' WHERE ' . $this->table_id . ' = ' . (int)$ID; 
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $this->db->CompleteTrans(); 
        return TRUE;
    }

    /**
    * Change items position.
    *
    * @param integer $id1 first item ID
    * @param integer $id2 second item ID
    * @return bool TRUE if successful, FALSE otherwise.
    */
    function ChangePosition($id1, $id2) 
    {
        $node_info = $this->GetNodeInfo($id1);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId1, $rightId1, $level1, $parent1) = $node_info;
        $node_info = $this->GetNodeInfo($id2);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId2, $rightId2, $level2, $parent2) = $node_info;
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . ' = ' . $leftId2 .', '
        . $this->table_right . ' = ' . $rightId2 .', '
        . $this->table_level . ' = ' . $level2 .', '
        . $this->table_parent . ' = ' . $parent2 .' '
        . 'WHERE ' . $this->table_id . ' = ' . (int)$id1;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . ' = ' . $leftId1 .', '
        . $this->table_right . ' = ' . $rightId1 .', '
        . $this->table_level . ' = ' . $level1 .', '
        . $this->table_parent . ' = ' . $parent1 .' '
        . 'WHERE ' . $this->table_id . ' = ' . (int)$id2;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_parent . ' = -1 ' 
        . ' WHERE ' . $this->table_parent . ' = ' . (int)$id1;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_parent . ' = ' . $id1 
        . ' WHERE ' . $this->table_parent . ' = ' . (int)$id2;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_parent . ' = ' . $id2 
        . ' WHERE ' . $this->table_parent . ' = -1';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $this->db->CompleteTrans();
        return TRUE;
    }

    /**
    * Swapping nodes within the same level and limits of one parent with all its children: $id1 placed before or after $id2.
    *
    * @param integer $id1 first item ID
    * @param integer $id2 second item ID
    * @param string $position 'before' or 'after' $id2
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @return bool TRUE if successful, FALSE otherwise.
    */
    function ChangePositionAll($id1, $id2, $position = 'after', $condition = '') 
    {
        $node_info = $this->GetNodeInfo($id1);
        if (FALSE === $node_info) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('cant_change_position') : 'cant_change_position';
            return FALSE;
        }
        list($leftId1, $rightId1, $level1) = $node_info;
        $node_info = $this->GetNodeInfo($id2);
        if (FALSE === $node_info) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('cant_change_position') : 'cant_change_position';
            return FALSE;
        }
        list($leftId2, $rightId2, $level2) = $node_info;
        if ($level1 <> $level2) 
        {
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('cant_change_position') : 'cant_change_position';
            return FALSE;
        }
        if ('before' == $position) 
        {
            if ($leftId1 > $leftId2) 
            {
                $sql = 'UPDATE ' . $this->table . ' SET '
                . $this->table_right . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_right . ' - ' . ($leftId1 - $leftId2) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId2 . ' AND ' . ($leftId1 - 1) . ' THEN ' . $this->table_right . ' +  ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_right . ' END, '
                . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_left . ' - ' . ($leftId1 - $leftId2) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . $leftId2 . ' AND ' . ($leftId1 - 1) . ' THEN ' . $this->table_left . ' + ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_left . ' END '
                . 'WHERE ' . $this->table_left . ' BETWEEN ' . $leftId2 . ' AND ' . $rightId1;
            } else 
            {
                $sql = 'UPDATE ' . $this->table . ' SET '
                . $this->table_right . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_right . ' + ' . (($leftId2 - $leftId1) - ($rightId1 - $leftId1 + 1)) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId1 + 1) . ' AND ' . ($leftId2 - 1) . ' THEN ' . $this->table_right . ' - ' . (($rightId1 - $leftId1 + 1)) . ' ELSE ' . $this->table_right . ' END, '
                . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_left . ' + ' . (($leftId2 - $leftId1) - ($rightId1 - $leftId1 + 1)) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId1 + 1) . ' AND ' . ($leftId2 - 1) . ' THEN ' . $this->table_left . ' - ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_left . ' END '
                . 'WHERE ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . ($leftId2 - 1);
            }
        }
        if ('after' == $position) 
        {
            if ($leftId1 > $leftId2) 
            {
                $sql = 'UPDATE ' . $this->table . ' SET '
                . $this->table_right . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_right . ' - ' . ($leftId1 - $leftId2 - ($rightId2 - $leftId2 + 1)) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId2 + 1) . ' AND ' . ($leftId1 - 1) . ' THEN ' . $this->table_right . ' +  ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_right . ' END, '
                . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_left . ' - ' . ($leftId1 - $leftId2 - ($rightId2 - $leftId2 + 1)) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId2 + 1) . ' AND ' . ($leftId1 - 1) . ' THEN ' . $this->table_left . ' + ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_left . ' END '
                . 'WHERE ' . $this->table_left . ' BETWEEN ' . ($rightId2 + 1) . ' AND ' . $rightId1;
            } else 
            {
                $sql = 'UPDATE ' . $this->table . ' SET '
                . $this->table_right . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_right . ' + ' . ($rightId2 - $rightId1) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId1 + 1) . ' AND ' . $rightId2 . ' THEN ' . $this->table_right . ' - ' . (($rightId1 - $leftId1 + 1)) . ' ELSE ' . $this->table_right . ' END, '
                . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId1 . ' THEN ' . $this->table_left . ' + ' . ($rightId2 - $rightId1) . ' '
                . 'WHEN ' . $this->table_left . ' BETWEEN ' . ($rightId1 + 1) . ' AND ' . $rightId2 . ' THEN ' . $this->table_left . ' - ' . ($rightId1 - $leftId1 + 1) . ' ELSE ' . $this->table_left . ' END '
                . 'WHERE ' . $this->table_left . ' BETWEEN ' . $leftId1 . ' AND ' . $rightId2;
            }
        }
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition);
        }
        $sql .= $condition;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $this->db->CompleteTrans();
        return TRUE;
    }

    /**
    * Delete element with number $id from the tree wihtout deleting it's children.
    *
    * @param integer $ID Number of element
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @return id if successful, FALSE otherwise.
    */
    function Delete($ID, $condition = '') 
    {
        $node_info = $this->GetNodeInfo($ID);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition);
        }
        $sql = 'DELETE FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = ' . (int)$ID;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_level . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_level . ' - 1 ELSE ' . $this->table_level . ' END, '
        . $this->table_right . ' = CASE WHEN ' . $this->table_right . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_right . ' - 1 '
        . 'WHEN ' . $this->table_right . ' > ' . $rightId . ' THEN ' . $this->table_right . ' - 2 ELSE ' . $this->table_right . ' END, '
        . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId . ' THEN ' . $this->table_left . ' - 1 '
        . 'WHEN ' . $this->table_left . ' > ' . $rightId . ' THEN ' . $this->table_left . ' - 2 ELSE ' . $this->table_left . ' END '
        . 'WHERE ' . $this->table_right . ' > ' . $leftId;
        $sql .= $condition;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_parent . ' = ' . $parent 
        . ' WHERE ' . $this->table_parent . ' = ' . (int)$ID;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $this->db->CompleteTrans();
        return $id;
    }

    /**
    * Delete element with number $ID from the tree and all it childret.
    *
    * @param integer $ID Number of element
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @return res с id удаляемых ел-тов if successful, FALSE otherwise.
    */
    function DeleteAll($ID, $condition = '') 
    {
    	$id_field = array($this->table_id);
    	$child = $this->GetChildTree($ID, $id_field, 1);
        $node_info = $this->GetNodeInfo($ID);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId) = $node_info;
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition);
        }
        $sql = 'DELETE FROM ' . $this->table . ' WHERE ' . $this->table_left . ' BETWEEN ' . $leftId . ' AND ' . $rightId;
        $this->db->StartTrans();
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $deltaId = (($rightId - $leftId) + 1);
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' > ' . $leftId.' THEN ' . $this->table_left . ' - ' . $deltaId . ' ELSE ' . $this->table_left . ' END, '
        . $this->table_right . ' = CASE WHEN ' . $this->table_right . ' > ' . $leftId . ' THEN ' . $this->table_right . ' - ' . $deltaId . ' ELSE ' . $this->table_right . ' END '
        . 'WHERE ' . $this->table_right . ' > ' . $rightId;
        $sql .= $condition;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            $this->db->FailTrans();
            return FALSE;
        }
        $this->db->CompleteTrans();
        return $child;
    }
    
    /* Функция возвращает всё подчинённое дерево без самого родителя ($roditel=0), с родителем ($roditel=не 0)
       $id - id родителя
       возвращает результат запроса со всеми данными узлов подчинённого дерева                                */
    function GetChildTree($id, $fields, $roditel=0)
    {
    	$node_info = $this->GetNodeInfo($id);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId) = $node_info;
        $sql = $this->PrepareSel($fields);
        if ($roditel==0)
    	{
        	$sql .= ' WHERE ' . $this->table_left . ' > ' . $leftId 
        	. ' and ' . $this->table_right . ' < ' . $rightId 
        	. ' order by ' . $this->table_left; 
    	} else 
    	{
    		$sql .= ' WHERE ' . $this->table_left . ' >= ' . $leftId 
        	. ' and ' . $this->table_right . ' <= ' . $rightId 
        	. ' order by ' . $this->table_left;
    	}
    	$res = $this->db->Execute($sql);
    	if (FALSE === $res) 
    	{
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
    	$this->res = $res;
        return $res;
    }

    /**
    * Returns all elements of the tree sortet by left.
    *
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $fields needed fields (if is): array('filed1 name', 'filed2 name', etc)
    * @param integer $cache Recordset is cached for $cache microseconds
    * @return array needed fields
    */
    function Full($fields, $condition = '')
    {
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition, TRUE);
        }
        $sql = $this->PrepareSel($fields);
        $sql .= $condition;
        $sql .= ' ORDER BY ' . $this->table_left;

        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }

    /**
    * Returns all elements of a branch starting from an element with number $ID.
    *
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $fields needed fields (if is): array('filed1 name', 'filed2 name', etc)
    * @param integer $cache Recordset is cached for $cache microseconds
    * @param integer $ID Node unique id
    * @return array - [0] => array(id, left, right, level, additional fields), [1] => array(...), etc.
    */
    function Branch($ID, $fields, $condition = '') 
    {
        if (is_array($fields)) 
        {
            $fields = 'A.' . implode(', A.', $fields);
        } else 
        {
            $fields = 'A.*';
        }
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition, FALSE, 'A.');
        }
        $sql = 'SELECT ' . $fields . ', CASE WHEN A.' . $this->table_left . ' + 1 < A.' . $this->table_right . ' THEN 1 ELSE 0 END AS nflag FROM ' . $this->table . ' A, ' . $this->table . ' B WHERE B.' . $this->table_id . ' = ' . (int)$ID . ' AND A.' . $this->table_left . ' >= B.' . $this->table_left . ' AND A.' . $this->table_right . ' <= B.' . $this->table_right;
        $sql .= $condition;
        $sql .= ' ORDER BY A.' . $this->table_left;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }

    /**
    * Returns all parents of element with number $ID.
    *
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $fields needed fields (if is): array('filed1 name', 'filed2 name', etc)
    * @param integer $cache Recordset is cached for $cache microseconds
    * @param integer $ID Node unique id
    * @return array - [0] => array(id, left, right, level, additional fields), [1] => array(...), etc.
    */
    function Parents($ID, $fields, $condition = '')
    {
        if (is_array($fields)) 
        {
            $fields = 'A.' . implode(', A.', $fields);
        } else 
        {
            $fields = 'A.*';
        }
        if (!empty($condition)) 
        {
            $condition = $this->_PrepareCondition($condition, FALSE, 'A.');
        }
        $sql = 'SELECT ' . $fields . ', CASE WHEN A.' . $this->table_left . ' + 1 < A.' . $this->table_right . ' THEN 1 ELSE 0 END AS nflag FROM ' . $this->table . ' A, ' . $this->table . ' B WHERE B.' . $this->table_id . ' = ' . (int)$ID . ' AND B.' . $this->table_left . ' BETWEEN A.' . $this->table_left . ' AND A.' . $this->table_right;
        $sql .= $condition;
        $sql .= ' ORDER BY A.' . $this->table_left;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }

    /**
    * Returns a slightly opened tree from an element with number $ID.
    *
    * @param array $condition Array structure: array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc where array key - condition (AND, OR, etc), value - condition string
    * @param array $fields needed fields (if is): array('filed1 name', 'filed2 name', etc)
    * @param integer $cache Recordset is cached for $cache microseconds
    * @param integer $ID Node unique id
    * @return array - [0] => array(id, left, right, level, additional fields), [1] => array(...), etc.
    */
    function Ajar($ID, $fields, $condition = '') 
    {
        if (is_array($fields)) 
        {
            $fields = 'A.' . implode(', A.', $fields);
        } else 
        {
            $fields = 'A.*';
        }
        $condition1 = '';
        if (!empty($condition)) 
        {
            $condition1 = $this->_PrepareCondition($condition, FALSE, 'B.');
        }
        $sql = 'SELECT A.' . $this->table_left . ', A.' . $this->table_right . ', A.' . $this->table_level . ' FROM ' . $this->table . ' A, ' . $this->table . ' B '
        . 'WHERE B.' . $this->table_id . ' = ' . (int)$ID . ' AND B.' . $this->table_left . ' BETWEEN A.' . $this->table_left . ' AND A.' . $this->table_right;
        $sql .= $condition1;
        $sql .= ' ORDER BY A.' . $this->table_left;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        if (0 == $res->RecordCount()) 
        {
            $this->ERRORS_MES[] = _('no_element_in_tree');
            return FALSE;
        }
        $alen = $res->RecordCount();
        $i = 0;
        if (is_array($fields)) 
        {
            $fields = implode(', ', $fields);
        } else 
        {
            $fields = '*';
        }
        if (!empty($condition)) 
        {
            $condition1 = $this->_PrepareCondition($condition, FALSE);
        }
        $sql = 'SELECT ' . $fields . ' FROM ' . $this->table . ' WHERE (' . $this->table_level . ' = 1';
        while ($row = $res->FetchRow()) 
        {
            if ((++$i == $alen) && ($row[$this->table_left] + 1) == $row[$this->table_right]) 
            {
                break;
            }
            $sql .= ' OR (' . $this->table_level . ' = ' . ($row[$this->table_level] + 1)
            . ' AND ' . $this->table_left . ' > ' . $row[$this->table_left]
            . ' AND ' . $this->table_right . ' < ' . $row[$this->table_right] . ')';
        }
        $sql .= ') ' . $condition1;
        $sql .= ' ORDER BY ' . $this->table_left;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }

    /**
    * Returns amount of lines in result.
    *
    * @return integer
    */
    function RecordCount() 
    {
        return $this->res->RecordCount();
    }

    /**
    * Returns the current row.
    *
    * @return array
    */
    function NextRow($opt=MYSQL_BOTH) 
    {
        return $this->res->FetchRow($opt);
    }

    /**
    * Transform array with conditions to SQL query
    * Array structure:
    * array('and' => array('id = 0', 'id2 >= 3'), 'or' => array('sec = \'www\'', 'sec2 <> \'erere\'')), etc
    * where array key - condition (AND, OR, etc), value - condition string.
    *
    * @param array $condition
    * @param string $prefix
    * @param bool $where - True - yes, flase - not
    * @return string
    */
    function _PrepareCondition($condition, $where = FALSE, $prefix = '') 
    {
        if (!is_array($condition)) 
        {
            return $condition;
        }
        $sql = ' ';
        if (TRUE === $where) 
        {
            $sql .= 'WHERE ' . $prefix;
        }
        $keys = array_keys($condition);
        for ($i = 0;$i < count($keys);$i++) 
        {
            if (FALSE === $where || (TRUE === $where && $i > 0)) 
            {
                $sql .= ' ' . strtoupper($keys[$i]) . ' ' . $prefix;
            }
            $sql .= implode(' ' . strtoupper($keys[$i]) . ' ' . $prefix, $condition[$keys[$i]]);
        }
        return $sql;
    }
    
    function PrepareSel($fields)
    {
    	if (is_array($fields)) 
        {
            $fields = implode(', ', $fields);
        } else 
        {
            $fields = '*';
        }
        $sql = 'SELECT ' . $fields . ' FROM ' . $this->table;
        return $sql;
    }
	
    function PrepareSelLeftJoin($fields, $leftjoin)
    {
    	if (is_array($fields))
        {
            $fields = implode(', ', $fields);
        } else
        {
            $fields = '*';
        }

		if (count($leftjoin["table"])>0)
        {
            for ($i=0; $i<count($leftjoin["table"]); $i++)
            {
                $skoba_close=' '.str_repeat(")",$i);
                if ($i>0)
                {
                    $str_leftjoin = $str_leftjoin.$skoba_close.' LEFT JOIN '.$leftjoin["table"][$i].' ON '.$leftjoin["table"][$i-1].'.'.$leftjoin["field"][$i].'='.$leftjoin["table"][$i].'.'.$leftjoin["field"][$i];
                }
                else
                {
                   $str_leftjoin = $str_leftjoin.$skoba_close.' LEFT JOIN '.$leftjoin["table"][$i].' ON '.$this->table.'.'.$leftjoin["field"][$i].'='.$leftjoin["table"][$i].'.'.$leftjoin["field"][$i];
                }
            }
        }
        else
        {
            $str_leftjoin = '';
        }

        $skoba_open=str_repeat("(",count($leftjoin["table"])-1);

        $sql = 'SELECT ' . $fields . ' FROM ' . $skoba_open. $this->table . $str_leftjoin;
        return $sql;
    }
    
    function GetChildsInfoAll($id, $fields)
    {
    	$sql = $this->PrepareSel($fields);
    	$sql .= ' WHERE '. $this->table_parent . ' = ' . $id;
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }

	function GetChildsCategoryName($id, $fields, $leftjoin)
    {
    	$sql = $this->PrepareSelLeftJoin($fields, $leftjoin);
    	$sql .= ' WHERE '. $this->table . '.' . $this->table_parent . ' = ' . $id;

    	$res = $this->db->Execute($sql);
        if (FALSE === $res)
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }

        if ($res->RecordCount()>0)
        {
            $kol_fields=$res->FieldNums();
            while ($result_cat = $res->FetchArray())
            {
                for ($i=0;$i<$kol_fields;$i++)
                {
                    $field_name=$res->FieldName($i);
                    $result_mass[$field_name][]=$result_cat[$i];
                }
            }
        }

        $this->res = $res;
        return $result_mass;
    }
    
    /* Функция проверки - левый ключ всегда меньше правого
       возвращает результат запроса, 
       если он пустой, то дерево правильно, если неправильно, то возвращает id неправильных строк  */ 
    function CheckLeftMinRight()
    {
    	$sql = 'SELECT ' . $this->table_id . ' FROM ' . $this->table
    	. ' WHERE ' . $this->table_left . ' >= ' . $this->table_right;
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }
    
    /* Функция проверки - минимальный левый всегда = 1, максимальный правый = 2*количество ел-тов дерева
       возвращает результат запроса (массив 0-кол-во ел-тов в дереве, 1-минимальный левый, 2-максимальный правый) */                                            
    function CheckMinLeftMaxRight()
    {
    	$sql = 'SELECT count(' . $this->table_id . '), min(' . $this->table_left . '), max(' . $this->table_right . ') FROM ' . $this->table;
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }
    
    /* Функция проверки - разница между правым и левым ключом всегда нечётное число
       возвращает результат запроса, 
       если он пустой, то дерево правильно, если неправильно, то возвращает id неправильных строк */
    function CheckRightMinusLeft()
    {
    	$sql = 'SELECT ' . $this->table_id . ' FROM ' . $this->table 
    	. ' WHERE mod((' . $this->table_right . '-' . $this->table_left . '),2)=0';
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }
    
    /* Функция проверки - если уровень узла нечётное число, то и левый ключ нечётный, если чётное, то и левый чётный 
       возвращает результат запроса, 
       если он пустой, то дерево правильно, если неправильно, то возвращает id неправильных строк                       */
    function CheckLeftAndLevel()
    {
    	$sql = 'SELECT ' . $this->table_id . ' FROM ' . $this->table 
    	. ' WHERE mod((' . $this->table_left . '-' . $this->table_level . '+2),2)=1';
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;
    }
    
    /* Функция проверки - ключи всегда уникальны, вне зависимости от того правый он или левый 
       возвращает результат запроса, 
       если он пустой, то дерево правильно, если неправильно, то возвращает id неправильных строк  */
    function CheckUniqLeftRight()
    {
    	$sql = 'SELECT t1.' .$this->table_id . ', count(t1.' .$this->table_id . ') as rep, max(t3.' .$this->table_right . ') as max_right FROM '
    	        . $this->table . ' as t1, ' . $this->table . ' as t2, ' . $this->table . ' as t3 WHERE t1.' .$this->table_left . ' <> t2.' .$this->table_left
    	        . ' and t1.' .$this->table_left . ' <> t2.' .$this->table_right . ' and t1.' .$this->table_right . ' <> t2.' .$this->table_left
    	        . ' and t1.' .$this->table_right . ' <> t2.' .$this->table_right
    	        . ' group by t1.' .$this->table_id . ' having max_right <> sqrt(4*rep +1)+1';
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $this->res = $res;
        return $res;        
    }
    
    /* 
    Функция для вставки первого ел-та дерева при его востановлении. 
    Входящие: 
		$id - id с которым нужно вставить ел-т в таблицу;
    	$data - массив с полями и их значениями для добавляемой записи в виде array('filed name' => 'значение', ...).  		
    Возвращает:
		$id - id вставленного ел-та;
		или FALSE.
	*/
    function InsertFirstResurection($id, $data = array())
    {
    	unset($data[$this->table_left], $data[$this->table_right], $data[$this->table_level], $data[$this->table_parent]);
    	foreach ($data as $k => $v) 
        {
        	if ($v == '')
        	{
        		unset($data[$k]);
        	}
		}
    	if (!empty($data)) 
        {
            $fld_names = implode(', ', array_keys($data)) . ', ';
            $fld_values = '\'' . implode('\', \'', array_values($data)) . '\', ';
        }
        $fld_names .= $this->table_left . ', ' . $this->table_right . ', ' . $this->table_level . ', ' . $this->table_parent;
        $fld_values .= '1, 2, 1, 0';
        $sql = 'INSERT INTO ' . $this->table . ' (' . $this->table_id . ', ' . $fld_names . ') VALUES (' . $id . ', ' . $fld_values . ')';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        return $id;
    }
    
    
    /* 
    Функция для вставки соседнего ел-та дерева при его востановлении. 
    Входящие: 
		$id - id с которым нужно вставить ел-т в таблицу;
		$id_near - id ел-та, рядом с которым нужно вставить ел-т;
    	$data - массив с полями и их значениями для добавляемой записи в виде array('filed name' => 'значение', ...).  		
    Возвращает:
		$id - id вставленного ел-та;
		или FALSE.
	*/
    function InsertNearResurection($id, $id_near, $data = array())
    {
    	$node_info = $this->GetNodeInfo($id_near);
        if (FALSE === $node_info)
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        $data[$this->table_left] = ($rightId + 1);
        $data[$this->table_right] = ($rightId + 2);
        $data[$this->table_level] = ($level);
        $data[$this->table_parent] = ($parent);
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . ' = CASE WHEN ' . $this->table_left . ' > ' . $rightId . ' THEN ' . $this->table_left . ' + 2 ELSE ' . $this->table_left . ' END, '
        . $this->table_right . ' = CASE WHEN ' . $this->table_right . '> ' . $rightId . ' THEN ' . $this->table_right . ' + 2 ELSE ' . $this->table_right . ' END '
        . 'WHERE ' . $this->table_right . ' > ' . $rightId;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = -1';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $data[$this->table_id] = $id;
        $sql = $this->db->GetInsertSQL($res, $data);
        if (!empty($sql)) 
        {
            $res = $this->db->Execute($sql);
            if (FALSE === $res) 
            {
                $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
                $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
                return FALSE;
            }
        }
        return $id;
    }
    
    
    /* 
    Функция для вставки ел-та дерева под родителя при его востановлении. 
    Входящие: 
		$id - id с которым нужно вставить ел-т в таблицу;
		$id_parent - id ел-та, под которым нужно вставить ел-т;
    	$data - массив с полями и их значениями для добавляемой записи в виде array('filed name' => 'значение', ...).  		
    Возвращает:
		$id - id вставленного ел-та;
		или FALSE.
	*/
    function InsertResurection($id, $id_parent, $data = array())
    {
    	$node_info = $this->GetNodeInfo($id_parent);
        if (FALSE === $node_info) 
        {
            return FALSE;
        }
        list($leftId, $rightId, $level, $parent) = $node_info;
        $data[$this->table_left] = $rightId;
        $data[$this->table_right] = ($rightId + 1);
        $data[$this->table_level] = ($level + 1);
        $data[$this->table_parent] = $id_parent;
        $sql = 'UPDATE ' . $this->table . ' SET '
        . $this->table_left . '=CASE WHEN ' . $this->table_left . '>' . $rightId . ' THEN ' . $this->table_left . '+2 ELSE ' . $this->table_left . ' END, '
        . $this->table_right . '=CASE WHEN ' . $this->table_right . '>=' . $rightId . ' THEN ' . $this->table_right . '+2 ELSE ' . $this->table_right . ' END '
        . 'WHERE ' . $this->table_right . '>=' . $rightId;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_id . ' = -1';
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
            $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $data[$this->table_id] = $id; 
        $sql = $this->db->GetInsertSQL($res, $data);
        if (!empty($sql)) 
        {
            $res = $this->db->Execute($sql);
            if (FALSE === $res) 
            {
                $this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
                $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
                return FALSE;
            }
        }
        return $id;
    }
    
    /* 
    Функция рекурсивная для нахождения детей и их вставки под родителя при востановлении дерева. 
    Входящие: 
		$id_parent - id ел-та, под которым нужно вставить ел-т (детей которого нужно найти);
    	$id_max - максимальный id в таблице (+1 на всякий случай).  		
    Возвращает:
		TRUE или FALSE.
	*/
    function RecurWork($id_parent, $id_max)
    {
    	$sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_parent . ' = ' . $id_parent;
		$res = $this->db->Execute($sql);
    	if (FALSE === $res) 
    	{
        	$this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
        	$this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
        	return FALSE;
    	}
    	$child_count = $res->RecordCount();
    	if(0 == $child_count)
    	{
    		$this->ERRORS_MES[] = extension_loaded('gettext') ? _('no_child_in_parent') : 'no_child_in_parent';
            return TRUE;
    	}
    	for($i=0; $i<$child_count; $i++)
    	{
    		$data = $res->FetchRow(MYSQL_ASSOC);
    		$id = $data[$this->table_id] - $id_max;
    		unset($data[$this->table_id]);
    		$ins_ok = $this->InsertResurection($id, $id_parent, $data);
    		if (FALSE === $ins_ok) 
    		{
    			$this->ERRORS_MES[] = extension_loaded('gettext') ? _('not_inserted') : 'not_inserted';
        		return FALSE;
    		}
    		$rec_ok = $this->RecurWork($id, $id_max);
    		if (FALSE === $rec_ok) 
    		{
    			$this->ERRORS_MES[] = extension_loaded('gettext') ? _('next_recurs_function_not_ok') : 'next_recurs_function_not_ok';
        		return FALSE;
    		} 
    	}
    	return TRUE;
    }
    
    
    /* 
    Функция востановления дерева. 
    Входящие: 
    Возвращает:
		TRUE или FALSE.
	*/
    function Resurection()
    {
    	//Находим максимальный id в таблице.
    	$sql = 'SELECT max(' . $this->table_id . ') FROM ' . $this->table;
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
        	$this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        $data = $res->FetchRow();
        unset($res);
        $id_max = $data[0] + 1;
        unset($data);
        //Нашли максимальный id в таблице (+1 на всякий случай), $id_max.
        
        //Сдвигаем все ел-ты вниз на $id_max, освобождаем место для правильных ел-тов, которые будем вставлять.
        $sql = 'UPDATE ' . $this->table . ' SET ' . $this->table_id . ' = ' . $this->table_id . ' + ' . $id_max;
        $res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
        	$this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        //Сдвинули.
        
        //Ищем корневые ел-ты (родитель = 0).
        $sql = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->table_parent . ' = ' . 0;
		$res = $this->db->Execute($sql);
    	if (FALSE === $res) 
    	{
        	$this->ERRORS[] = array(2, 'SQL query error.', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
        	$this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
        	return FALSE;
    	}
    	$root_count = $res->RecordCount();
    	if(0 == $root_count)
    	{
    		$this->ERRORS_MES[] = extension_loaded('gettext') ? _('no_root_element_in_tree') : 'no_root_element_in_tree';
            return FALSE;
    	}
    	//Нашли.
    	
    	//Вставляем первый корень.
    	$data = $res->FetchRow(MYSQL_ASSOC);
    	$id = $data[$this->table_id] - $id_max;
    	unset($data[$this->table_id]);
    	$ins_first_ok = $this->InsertFirstResurection($id, $data);
    	if (FALSE === $ins_first_ok) 
    	{
    		$this->ERRORS_MES[] = extension_loaded('gettext') ? _('first_not_inserted') : 'first_not_inserted';
        	return FALSE;
    	}
    	//Вставили первый корень.
    	
    	//Вставляем всю ветку под первым корнем.
    	$rec_after_first_ok = $this->RecurWork($id, $id_max);	
    	if (FALSE === $rec_after_first_ok) 
    	{
    		$this->ERRORS_MES[] = extension_loaded('gettext') ? _('first_recurs_function_not_ok') : 'first_recurs_function_not_ok';
        	return FALSE;
    	}
    	//Вставили всю ветку под первым корнем.
    	
    	//Вставляем остальные корни рядом с первым.
    	$id_near = $ins_first_ok;
    	for($i=1; $i<$root_count; $i++)
    	{
    		$data = $res->FetchRow(MYSQL_ASSOC);
    		$id = $data[$this->table_id] - $id_max;
    		unset($data[$this->table_id]);
    		$ins_near_ok = $this->InsertNearResurection($id, $id_near, $data);
   		    if (FALSE === $ins_near_ok) 
    		{
    			$this->ERRORS_MES[] = extension_loaded('gettext') ? _('near_not_inserted') : 'near_not_inserted';
        		return FALSE;
    		}
    		//Вставили один соседний корень.
    		
    		//Вставляем всю ветку под этим соседним корнем.
    		$rec_after_near_ok = $this->RecurWork($id, $id_max);
    		if (FALSE === $rec_after_near_ok) 
    		{
    			$this->ERRORS_MES[] = extension_loaded('gettext') ? _('near_recurs_function_not_ok') : 'near_recurs_function_not_ok';
        		return FALSE;
    		}
    		//Вставили всю ветку под этим соседним корнем.
    		$id_near = $ins_near_ok;
    	}
    	//Вставили остальные корни рядом с первым (и все их ветки заодно).
    	
    	//Удаляем старые записи.
    	$sql = 'DELETE FROM ' . $this->table . ' WHERE ' . $this->table_id . ' > ' . $id_max;
    	$res = $this->db->Execute($sql);
        if (FALSE === $res) 
        {
        	$this->ERRORS[] = array(2, 'SQL query error', __FILE__ . '::' . __CLASS__ . '::' . __FUNCTION__ . '::' . __LINE__, 'SQL QUERY: ' . $sql, 'SQL ERROR: ' . $this->db->ErrorMsg());
            $this->ERRORS_MES[] = extension_loaded('gettext') ? _('internal_error') : 'internal_error';
            return FALSE;
        }
        //Удалили старые записи. 
    	return TRUE;
    }
    
    /* Функция проверки всего дерева 
       возвращает код ошибки в виде 020406  */
    function CheckTree()
    {
    	$this->tree_error_code = '0';
    	$check_LeftMinRight = $this->CheckLeftMinRight();
    	$check_MinLeftMaxRight = $this->CheckMinLeftMaxRight();
    	$check_RightMinusLeft = $this->CheckRightMinusLeft();
    	$check_LeftAndLevel = $this->CheckLeftAndLevel();
    	$check_UniqLeftRight = $this->CheckUniqLeftRight();
    	$check_MinLeftMaxRight_array = $check_MinLeftMaxRight->FetchRow();
    	if($check_LeftMinRight->RecordCount()<>0)
    		$this->tree_error_code = '1';
    	if ($check_MinLeftMaxRight_array[1]<>1)
		    $this->tree_error_code = $this->tree_error_code.'2';
		else 
		    $this->tree_error_code = $this->tree_error_code.'0';
        if ($check_MinLeftMaxRight_array[2]<>(2*$check_MinLeftMaxRight_array[0]))
			$this->tree_error_code = $this->tree_error_code.'3'; 
		else 
		    $this->tree_error_code = $this->tree_error_code.'0';	
		if ($check_RightMinusLeft->RecordCount()<>0)
			$this->tree_error_code = $this->tree_error_code.'4';
		else 
		    $this->tree_error_code = $this->tree_error_code.'0';	
		if ($check_LeftAndLevel->RecordCount()<>0)
			$this->tree_error_code = $this->tree_error_code.'5';
		else 
		    $this->tree_error_code = $this->tree_error_code.'0';	
		if ($check_UniqLeftRight->RecordCount()<>0)
			$this->tree_error_code = $this->tree_error_code.'6';
		else 
		    $this->tree_error_code = $this->tree_error_code.'0';
		return $this->tree_error_code;    					   	
    }
}
?>